﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.UI;

using TechED_Demo.Utility.UI;

namespace TechED_Demo.Utility.ExtenMethods
{
    public static class ExtenWebUI
    {
        /// <summary>
        /// 将对象中指定属性（特殊文本框指定的属性名）的值写入特殊文本框
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="txtControl">特殊文本框</param>
        /// <param name="obj">对象实体</param>
        public static void ReadPropertyToTextBox<T>(this EntityTextBox txtControl, T obj)
        {
            if (txtControl.EntityType == typeof(T).FullName)
            {
                object propertyValue = typeof(T).GetProperty(txtControl.EntityPropertyName).GetValue(obj, null);
                string propertyFormat = txtControl.EntityPropertyValueFormat;
                if (propertyValue != null)
                {
                    if (string.IsNullOrEmpty(propertyFormat))
                    {
                        txtControl.Text = propertyValue.ToString();
                    }
                    else
                    {
                        txtControl.Text = string.Format(propertyFormat, propertyValue);
                    }
                }
            }
        }

        /// <summary>
        /// 将对象中指定属性（特殊文本框指定了属性名）的值按指定格式写入特殊文本框
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="txtControl">特殊文本框</param>
        /// <param name="obj">对象实体</param>
        /// <param name="format">格式化字符串</param>
        public static void ReadPropertyToTextBox<T>(this EntityTextBox txtControl, T obj,string format)
        {
            if (txtControl.EntityType == typeof(T).FullName)
            {
                object propertyValue = typeof(T).GetProperty(txtControl.EntityPropertyName).GetValue(obj, null);
                if (propertyValue != null)
                {
                    txtControl.Text = string.Format(format,propertyValue);
                }
            }
        }

        /// <summary>
        /// 将对象的所有属性值读到对应的特殊文本框中
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="currentControl">特殊文本框</param>
        /// <param name="obj">对象实体</param>
        public static void ReadAllPropertyToTextBox<T>(this Control currentControl, T obj)
        {
            foreach (var item in currentControl.Controls)
            {
                if (item is EntityTextBox)
                {
                    EntityTextBox gtxt = (EntityTextBox)item;
                    gtxt.ReadPropertyToTextBox<T>(obj);
                }
            }
        }

        /// <summary>
        /// 将特殊文本框的值写入对象的对应属性（特殊文本框指定了属性名）中
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="txtControl">特殊文本框</param>
        /// <param name="obj">对象实体</param>
        public static void WritePropertyFromTextBox<T>(this EntityTextBox txtControl, ref T obj)
        {
            string propertyString = txtControl.Text.Trim();
            string propertyName = txtControl.EntityPropertyName;
            PropertyInfo propertyInfo = typeof(T).GetProperty(propertyName);
            if (!string.IsNullOrEmpty(propertyString))
            {

                object propertyValue;

                switch (propertyInfo.PropertyType.FullName)
                {
                    case "System.SByte":
                        propertyValue = sbyte.Parse(propertyString);
                        break;
                    case "System.Byte":
                        propertyValue = byte.Parse(propertyString);
                        break;

                    case "System.Int16":
                        propertyValue = short.Parse(propertyString);
                        break;
                    case "System.UInt16":
                        propertyValue = uint.Parse(propertyString);
                        break;

                    case "System.Int32":
                        propertyValue = int.Parse(propertyString);
                        break;
                    case "System.UInt32":
                        propertyValue = uint.Parse(propertyString);
                        break;

                    case "System.Int64":
                        propertyValue = long.Parse(propertyString);
                        break;
                    case "System.UInt64":
                        propertyValue = ulong.Parse(propertyString);
                        break;

                    case "System.Single":
                        propertyValue = float.Parse(propertyString);
                        break;

                    case "System.Double":
                        propertyValue = double.Parse(propertyString);
                        break;

                    case "System.Decimal":
                        propertyValue = decimal.Parse(propertyString);
                        break;

                    case "System.DateTime":
                        propertyValue = DateTime.Parse(propertyString);
                        break;

                    case "System.Guid":
                        propertyValue = new Guid(propertyString);
                        break;

                    default:
                        propertyValue = propertyString;
                        break;
                }

                propertyInfo.SetValue(obj, propertyValue, null);
            }
        }

        /// <summary>
        /// 将指定控件（页面）中的所有特殊文本框的值写入对象实体对应的属性中
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="currentControl">要查找的容器控件</param>
        /// <param name="obj">对象实例</param>
        public static void WriteAllPropertyFromTextBox<T>(this Control currentControl,ref T obj)
        {
            foreach (var item in currentControl.Controls)
            {
                if (item is EntityTextBox)
                {
                    EntityTextBox gtxt = (EntityTextBox)item;
                    if (gtxt.EntityType == typeof(T).FullName)
                    {
                        gtxt.WritePropertyFromTextBox<T>(ref obj);
                    }
                }
            }
        }


        /// <summary>
        /// 获取指定类型的对象实例，并将指定控件（页面）中的特殊文本框的值转化为对应的该对象的属性值
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="currentControl">要查找的容器控件（页面）</param>
        /// <returns>对象实例</returns>
        public static T WriteAllPropertyFromTextBox<T>(this Control currentControl) where T:new()
        {
            T entity = new T();
            currentControl.WriteAllPropertyFromTextBox<T>(ref entity);
            return entity;
        }
    }
}
