﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;


namespace TechED_Demo.Utility.UI
{
    [DefaultProperty("Text")]
    [ToolboxData("<{0}:SortLinkButton runat=server></{0}:SortLinkButton>")]
    public class SortLinkButton : CompositeControl
    {
        LinkButton _linkButton;
        SortOrderImage _sortOrderImage;

        private static readonly object EventClick = new object();
        private static readonly object EventCommand = new object();

        protected override void CreateChildControls()
        {
            Controls.Clear();

            _linkButton = new LinkButton();
            _sortOrderImage = new SortOrderImage();

            _linkButton.ID = "lbtnSort";
            _linkButton.Click += new EventHandler(on_click);

            _sortOrderImage.ID = "soiSort";

            Controls.Add(_linkButton);
            Controls.Add(_sortOrderImage);
        }

        #region 属性
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Localizable(true)]
        public string OnClientClick
        {
            get
            {
                EnsureChildControls();
                return _linkButton.OnClientClick;
            }
            set
            {
                _linkButton.OnClientClick = value;
            }
        }

        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Localizable(true)]
        public string Text
        {
            get
            {
                EnsureChildControls();
                return _linkButton.Text;
            }
            set
            {
                EnsureChildControls();
                _linkButton.Text = value;
            }
        }

        /// <summary>
        /// 排序的字段名称
        /// </summary>
        public string SortField
        {
            get
            {
                object text = ViewState["SortField"];
                if (text != null)
                {
                    return (string)text;
                }
                return "Id";
            }
            set
            {
                ViewState["SortField"] = value;
            }
        }

        /// <summary>
        /// 排序组用于区分其他排序内容
        /// </summary>
        public string SortGroup
        {
            get
            {
                object text = ViewState["SortGroup"];
                if (text != null)
                {
                    return (string)text;
                }
                return string.Empty;
            }
            set
            {
                ViewState["SortGroup"] = value;
            }
        }

        /// <summary>
        /// 排序状态
        /// </summary>
        public SortOrderType SortOrder
        {
            get
            {
                EnsureChildControls();
                return _sortOrderImage.SortOrder;
            }
            set
            {
                EnsureChildControls();
                _sortOrderImage.SortOrder = value;
            }
        }

        /// <summary>
        /// 是否显示图片，默认不显示
        /// </summary>
        public bool DisplayImage
        {
            get
            {
                EnsureChildControls();
                return _sortOrderImage.Display;
            }
            set
            {
                EnsureChildControls();
                _sortOrderImage.Display = value;
            }
        }
        #endregion

        #region 事件处理
        public event EventHandler Click
        {
            add
            {
                Events.AddHandler(EventClick, value);
            }
            remove
            {
                Events.RemoveHandler(EventClick, value);
            }
        }

        protected virtual void OnClick(EventArgs e)
        {
            EventHandler handler = (EventHandler)base.Events[EventClick];
            if (handler != null)
            {
                handler(this, e);
            }
        }

        private void on_click(object sender, EventArgs e)
        {
            OnClick(e);
        }

        #endregion

        #region 静态方法

        public static void ChangeSortOrder(SortLinkButton sortButton)
        {
            if (sortButton.SortOrder == SortOrderType.ASC)
            {
                sortButton.SortOrder = SortOrderType.DESC;
            }
            else
            {
                sortButton.SortOrder = SortOrderType.ASC;
            }
        }

        public static void DisplaySortImage(Control control, string sortField, string sortGroup,SortOrderType sortOrder)
        {
            foreach (var item in control.Controls)
            {
                if (item is SortLinkButton)
                {
                    var btn = (SortLinkButton)item;
                    if (btn.SortGroup == sortGroup && btn.SortField == sortField)
                    {
                        btn.SortOrder = sortOrder;
                        btn.DisplayImage = true;
                    }
                    else
                    {
                        btn.DisplayImage = false;
                    }
                }
            }
        }

        #endregion
    }
}
