﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using System.Text.RegularExpressions;

namespace TechED_Demo.Utility.UI
{
    public class SortSaver : WebControl
    {
        public SortSaver()
        {
            this.SortIndexQueryString = "sortid";
            this.SortOrderQueryString = "sortorder";
            this.SortFieldsSplitChar = ',';
        }

        /// <summary>
        /// 当前排序字段索引
        /// </summary>
        public int CurrentSortIndex {
            get
            {
                if (this.EnableUrl)
                {
                    int currentSortIndex = 0;
                    string sortId = Context.Request.QueryString[this.SortIndexQueryString];
                    int.TryParse(sortId, out currentSortIndex);
                    return currentSortIndex;
                }
                else
                {
                    object sortId = ViewState["CurrentSortIndex"];
                    if (sortId != null)
                    {
                        return (int)sortId;
                    }
                    return 0;
                }
            }
            set
            {
                if (this.EnableUrl)
                {
                    if (this.CurrentSortIndex != value)
                    {
                        string oldUrl = Context.Request.Url.AbsoluteUri;
                        string newUrl = AddQueryStringToUrl(oldUrl, this.SortIndexQueryString, value.ToString());
                        Context.Response.Redirect(newUrl, true);
                    }
                }
                else
                {
                    ViewState["CurrentSortIndex"] = value;
                }
            }
        }

        /// <summary>
        /// 当前排序字段名
        /// </summary>
        public string CurrentSortField
        {
            get
            {
                return SortFieldList[this.CurrentSortIndex];
            }
        }

        /// <summary>
        /// 当前排序字段列表
        /// </summary>
        public List<string> SortFieldList
        {
            get
            {
                return this.SortFields.Split(this.SortFieldsSplitChar).ToList<string>();
            }
        }

        /// <summary>
        /// 当前排序字段的字符串（如：Id,Title,……）
        /// </summary>
        public string SortFields { get; set; }

        /// <summary>
        /// 当前排序字段拆分用的字符（默认为：逗号(,)）
        /// </summary>
        public char SortFieldsSplitChar { get; set; }

        /// <summary>
        /// 当前排序类型
        /// </summary>
        public SortOrderType SortOrder
        {
            get
            {
                if (this.EnableUrl)
                {
                    byte sortTypeId = 0;
                    string sortId = Context.Request.QueryString[this.SortOrderQueryString];
                    byte.TryParse(sortId, out sortTypeId);
                    if (sortTypeId > 1)
                    {
                        sortTypeId = 0;
                    }
                    return (SortOrderType)sortTypeId;
                }
                else
                {
                    object sortType = ViewState["SortOrder"];
                    if (sortType != null)
                    {
                        return (SortOrderType)sortType;
                    }
                    return SortOrderType.ASC;
                }
            }
            set
            {
                if (this.EnableUrl)
                {
                    if (this.SortOrder != value)
                    {
                        string oldUrl = Context.Request.Url.AbsoluteUri;
                        string newUrl = AddQueryStringToUrl(oldUrl, this.SortOrderQueryString, ((byte)value).ToString());
                        Context.Response.Redirect(newUrl, true);
                    }
                }
                else
                {
                    ViewState["SortOrder"] = value;
                }
            }
        }

        /// <summary>
        /// 使用使用地址参数保存排序状态
        /// </summary>
        public bool EnableUrl { get; set; }

        /// <summary>
        /// 排序索引的Url参数名
        /// </summary>
        public string SortIndexQueryString { get; set; }

        /// <summary>
        /// 排序类型的Url参数名
        /// </summary>
        public string SortOrderQueryString { get; set; }

        /// <summary>
        /// 执行排序方法，并返回排序后的结果
        /// </summary>
        /// <typeparam name="T">排序后的结果</typeparam>
        /// <param name="fun">排序的无参方法</param>
        /// <returns></returns>
        public T Sort<T>(Func<T> fun)
        {
            return fun();
        }

        /// <summary>
        /// 将页面定向到当前制定的参数值上
        /// </summary>
        public void GotoUrl(int sortIndex,SortOrderType sortOrder)
        {
            if (this.EnableUrl)
            {
                string oldUrl = Context.Request.Url.AbsoluteUri;
                string newUrl = AddQueryStringToUrl(oldUrl, this.SortIndexQueryString, sortIndex.ToString());
                newUrl = AddQueryStringToUrl(newUrl, this.SortOrderQueryString, ((byte)sortOrder).ToString());

                Context.Response.Redirect(newUrl, true);
            }
        }


        public static string AddQueryStringToUrl(string url, string qname, string qvalue)
        {
            string r;
            string t = qname + "=";
            qvalue = HttpContext.Current.Server.UrlEncode(qvalue);
            string regexText = "([?]|[&])" + t + "([^&]*)";
            if (url.IndexOf(t) > -1)
            {
                Regex re = new Regex(regexText, RegexOptions.IgnoreCase);
                r = re.Replace(url, "$1" + t + qvalue);
            }
            else
            {
                char sep = '&';
                if (url.IndexOf("?") == -1)
                {
                    sep = '?';
                }
                r = url + sep + t + qvalue;
            }
            return r;
        }

    }
}
